package net.wanji.web.task;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.BeanMapUtils;
import net.wanji.databus.dao.mapper.CrossInfoMapper;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.vo.SignalStatusVO;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.web.entity.TCrossControlHist;
import net.wanji.web.entity.TDeviceStatusInfo;
import net.wanji.web.entity.TDeviceStatusLog;
import net.wanji.web.mapper.ControlHistMapper;
import net.wanji.web.mapper.TDeviceStatusLogMapper;
import net.wanji.web.mapper.TDeviceStatusMapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/01/03 10:43
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class SignalStatusTask {
    private final UtcFeignClients utcFeignClients;
    private final TDeviceStatusMapper tDeviceStatusMapper;
    private final ControlHistMapper controlHistMapper;
    private final TDeviceStatusLogMapper tDeviceStatusLogMapper;
    private final CrossInfoMapper crossInfoMapper;

    @Scheduled(fixedRate = 1 * 30 * 1000)
    @Transactional
    public void syncSignalStatus() {
        try {
            JsonViewObject jsonViewObject = utcFeignClients.runningStatusAlarm();
            if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
                log.error("定时任务同步信号机设备状态utcService调用异常");
                return;
            }
            List<Map<String, Object>> source = (List<Map<String, Object>>) jsonViewObject.getContent();
            List<SignalStatusVO> content = BeanMapUtils.mapsToObjects(source, SignalStatusVO.class);
            List<String> crossIdList = content.stream().map(SignalStatusVO::getCrossId).collect(Collectors.toList());
            List<CrossInfoPO> crossInfoPOList = new ArrayList<>();
            if (!CollectionUtils.isEmpty(crossIdList)) {
                crossInfoPOList = crossInfoMapper.selectByCrossIds(crossIdList);
            }
            List<CrossInfoPO> finalCrossInfoPOList = crossInfoPOList;
            content.forEach(signalStatusVO -> {
                String crossId = signalStatusVO.getCrossId();
                String name = null;
                for (CrossInfoPO crossInfoPO : finalCrossInfoPOList) {
                    if (StringUtils.equals(crossId, crossInfoPO.getId())) {
                        name = crossInfoPO.getName();
                        break;
                    }
                }
                Integer currentSignalStatus = signalStatusVO.getStatus() == null ? 0 : signalStatusVO.getStatus();
                Integer currentFaultType = signalStatusVO.getFaultType() == null ? 0 : signalStatusVO.getFaultType();
                // 信号机在线，但是故障，数据库状态为2
                if (currentSignalStatus == 1 && !Objects.equals(0, currentFaultType)) {
                    currentSignalStatus = 2;
                }

                // 更新数据库状态
                LambdaQueryWrapper<TDeviceStatusInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                if (StringUtils.isNotEmpty(crossId)) {
                    lambdaQueryWrapper.eq(TDeviceStatusInfo::getCode, crossId);
                    TDeviceStatusInfo tDeviceStatusInfo = tDeviceStatusMapper.selectOne(lambdaQueryWrapper);
                    if (Objects.nonNull(tDeviceStatusInfo)) {
                        if (isExucuteUpdate(currentSignalStatus, currentFaultType, tDeviceStatusInfo)) {
                            tDeviceStatusInfo.setStatus(currentSignalStatus);
                            tDeviceStatusInfo.setFaultType(currentFaultType);
                            tDeviceStatusInfo.setGmtCreate(new Date()); // 实时数据，创建和修改时间一致
                            tDeviceStatusMapper.updateById(tDeviceStatusInfo);
                            insertDeviceStatusLog(signalStatusVO, crossId, currentSignalStatus, currentFaultType, name);
                        }
                    } else {
                        TDeviceStatusInfo deviceStatusInfo = new TDeviceStatusInfo();
                        deviceStatusInfo.setCode(crossId);
                        deviceStatusInfo.setName(name);
                        deviceStatusInfo.setType(1);
                        deviceStatusInfo.setFaultType(currentFaultType);
                        deviceStatusInfo.setStatus(currentSignalStatus);
                        deviceStatusInfo.setGmtCreate(new Date());
                        deviceStatusInfo.setId(null);
                        tDeviceStatusMapper.insert(deviceStatusInfo);
                    }
                }
                insertControlHist(signalStatusVO);
            });
        } catch (Exception e) {
            log.error("定时任务同步信号机设备状态远程utcService调用异常", e);
        }
    }

    /**
     * 插入控制历史表
     *
     * @param signalStatusVO
     */
    private void insertControlHist(SignalStatusVO signalStatusVO) {
        // 更新控制历史表
        String crossId = signalStatusVO.getCrossId();
        Integer controlType = signalStatusVO.getControlType();
        TCrossControlHist tCrossControlHist = controlHistMapper.selectRecentOne(crossId);
        if (Objects.nonNull(tCrossControlHist) && !Objects.equals(controlType, tCrossControlHist.getType())) {
            controlHistMapper.insertOne(crossId, controlType, "未知");
        }
    }

    /**
     * 信号机状态插入设备日志表
     *
     * @param signalStatusVO
     * @param crossId
     * @param currentSignalStatus
     * @param currentFaultType
     */
    private void insertDeviceStatusLog(SignalStatusVO signalStatusVO, String crossId, Integer currentSignalStatus, Integer currentFaultType, String name) {
        // 插入日志表
        TDeviceStatusLog tDeviceStatusLogInfoDB = tDeviceStatusLogMapper.selectLastOne(crossId);
        if (Objects.nonNull(tDeviceStatusLogInfoDB) && tDeviceStatusLogInfoDB.getFaultType() != currentFaultType) {
            tDeviceStatusLogInfoDB.setStatus(currentSignalStatus);
            tDeviceStatusLogInfoDB.setFaultType(tDeviceStatusLogInfoDB.getFaultType());
            tDeviceStatusLogInfoDB.setGmtCreate(new Date());
            tDeviceStatusLogInfoDB.setId(null);
            tDeviceStatusLogMapper.insert(tDeviceStatusLogInfoDB);
        } else {
            TDeviceStatusLog tDeviceStatusLog = new TDeviceStatusLog();
            tDeviceStatusLog.setCode(crossId);
            tDeviceStatusLog.setName(name);
            tDeviceStatusLog.setType(1);
            tDeviceStatusLog.setStatus(signalStatusVO.getStatus());
            tDeviceStatusLog.setFaultType(signalStatusVO.getFaultType());
            tDeviceStatusLog.setGmtCreate(new Date());
            tDeviceStatusLog.setId(null);
            tDeviceStatusLogMapper.insert(tDeviceStatusLog);
        }
    }

    private static boolean isExucuteUpdate(Integer currentSignalStatus, Integer currentFaultType, TDeviceStatusInfo tDeviceStatusInfo) {
        // 故障自定义，当故障时，状态对应海康在线
        int status = tDeviceStatusInfo.getStatus() == 2 ? 1 : tDeviceStatusInfo.getStatus();
        return Objects.nonNull(tDeviceStatusInfo) && (currentSignalStatus != status || currentFaultType != tDeviceStatusInfo.getFaultType());
    }
}
